/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2001, 2016. All Rights Reserved.
 * (c) Copyright HCL Technologies Ltd. 2018. All Rights Reserved.
 * 
 * THE SAMPLE IS PROVIDED "AS IS," WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT OF
 * PATENTS, COPYRIGHTS OR OTHER PROPRIETARY RIGHTS OF OTHERS. NEITHER THE AUTHOR
 * NOR IBM/HCL WARRANT THAT THE FUNCTIONS, ROUTINES, AND DATA CONTAINED IN, OR
 * GENERATED AS A RESULT OF THE SAMPLES, WILL MEET YOUR REQUIREMENTS OR BE
 * ERROR-FREE. The entire risk related to the quality and performance of the
 * Samples is with you. In the event that there is any defect, you assume the
 * entire cost of all necessary services, repair or correction. IN NO EVENT WILL
 * THE AUTHOR OR IBM/HCL BE LIABLE TO YOU OR TO ANY THIRD PARTY FOR ANY DIRECT
 * OR INDIRECT DAMAGES (INCLUDING, WITHOUT LIMITATION, LOST PROFITS, LOST SAVINGS
 * OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES) ARISING OUT OF THE USE OR
 * INABILITY TO USE THE SAMPLE, EVEN IF THE AUTHOR OR IBM/HCL HAS BEEN ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGES. Some jurisdictions do not allow the
 * exclusion of implied warranties or the limitation or exclusion of liability
 * for incidental or consequential damages, so some of the above may not apply
 * to you.
 *******************************************************************************/
package com.samples.functions;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;

import com.ghc.ghTester.expressions.EvalUtils;
import com.ghc.ghTester.expressions.Function;

public class FormatDate extends Function {
	/**
	 * This function will return the date that we wish to format as a String.
	 */
	private Function m_fDate = null;

	/**
	 * This function will return the format of the input date.
	 */
	private Function m_fInputFormat = null;

	/**
	 * If not null, this function will return the required format.
	 */
	private Function m_fOutputFormat = null;

	public FormatDate() {
	}

	protected FormatDate(Function f1, Function f2, Function f3) {
		m_fDate = f1;
		m_fInputFormat = f2;
		m_fOutputFormat = f3;
	}

	@Override
	public Function create(int size, Vector params) {
		Function outputFormat = null;
		if (size == 3) {
			outputFormat = (Function) params.get(2);
		}
		return new FormatDate((Function) params.get(0), (Function) params
				.get(1), outputFormat);
	}

	@Override
	public Object evaluate(Object data) {
		String date = m_fDate.evaluateAsString(data);
		String inputFormat = m_fInputFormat.evaluateAsString(data);
		String outputFormat = "yyyy-MM-dd"; // Default format
		if (m_fOutputFormat != null) {
			outputFormat = m_fOutputFormat.evaluateAsString(data);
		}

		// When evaluating an expression, a string is only considered
		// to be a string when it is surrounded by double-qoutes. We
		// need to remove these before further processing.
		//
		// Even though dates and formats should be specified as strings,
		// make this requirement relaxed.
		if (EvalUtils.isString(date)) {
			date = EvalUtils.getString(date);
		}
		if (EvalUtils.isString(inputFormat)) {
			inputFormat = EvalUtils.getString(inputFormat);
		}
		if (EvalUtils.isString(outputFormat)) {
			outputFormat = EvalUtils.getString(outputFormat);
		}

		SimpleDateFormat inputFormatter = new SimpleDateFormat(inputFormat);
		String formattedDate = "";
		try {
			Date d = inputFormatter.parse(date);
			SimpleDateFormat outputFormatter = new SimpleDateFormat(
					outputFormat);
			formattedDate = outputFormatter.format(d);
		} catch (ParseException ex) {
			// Unable to parse the date. Do nothing here, the
			// function will return an empty string.
		}

		return "\"" + formattedDate + "\"";

	}
}
